import { Directive, ElementRef, HostListener, Optional, Renderer2, } from '@angular/core';
import { NgControl } from '@angular/forms';
import { Subject } from 'rxjs';
import { map, takeUntil } from 'rxjs/operators';
import { CardTypeService } from '../services/card-type.service';
import utils, { TIMEOUT_SECONDS } from '../utils';
import clearSpaces from '../utils/clear-spaces';
/**
 * Directive to format and validate a card CVC
 */
export class CardCvvDirective {
    constructor(_renderer, _elementRef, _ngControl, _cardTypeService) {
        this._renderer = _renderer;
        this._elementRef = _elementRef;
        this._ngControl = _ngControl;
        this._cardTypeService = _cardTypeService;
        this.onChange = () => { };
        this.onTouched = () => { };
        /**
         * attr for stop all subscribes
         */
        this._destroy$ = new Subject();
    }
    ngOnInit() {
        this._setupInput();
        this._cardTypeService.setCardCVVRef(this._elementRef);
    }
    ngAfterViewInit() {
        this._setCardCvcMask();
    }
    ngOnDestroy() {
        this._destroy$.next();
        this._destroy$.complete();
        if (this._rendererTimeout) {
            window === null || window === void 0 ? void 0 : window.clearTimeout(this._rendererTimeout);
        }
    }
    /**
     * get the current html input from element ref
     * @returns The native element from ref
     * @private
     */
    get _el() {
        return this._elementRef.nativeElement;
    }
    /**
     * get the control of the current element ref
     * @returns return the control of element
     * @private
     */
    get _control() {
        return this._ngControl.control;
    }
    /**
     * listener the input event from html input element
     * @param event - the keyboard event
     */
    onCardChanges(event) {
        this.writeValue(event);
    }
    /**
     * listener the input blur event to update input mask
     * @param event - the keyboard event
     */
    onCardBlur(event) {
        this.writeValue(event);
    }
    /**
     * Listener the keydown to check when backspace is pressed
     * @param event {KeyboardEvent} - The keyboard event
     */
    onKeyDown(event) {
        var _a, _b, _c, _d;
        const value = (_a = event === null || event === void 0 ? void 0 : event.target) === null || _a === void 0 ? void 0 : _a.value;
        if (event.key.toLowerCase() === utils.BACKSPACE_KEY_CODE && !utils.isValue(value)) {
            // focus into card number field case expiry date is empty
            (_d = (_c = (_b = this._cardTypeService) === null || _b === void 0 ? void 0 : _b.cardExpiryRef) === null || _c === void 0 ? void 0 : _c.nativeElement) === null || _d === void 0 ? void 0 : _d.focus();
        }
    }
    /**
     * Writes a new value to the element.
     * @param rawValue - value from input
     */
    writeValue(rawValue) {
        var _a;
        const cvcValue = ((_a = rawValue.target) === null || _a === void 0 ? void 0 : _a.value) || '';
        const cvcValueFormatted = clearSpaces(cvcValue);
        const reg = new RegExp(`([0-9]{${this._cvvMaskLength}})`, 'g');
        const cvcmask = cvcValueFormatted === null || cvcValueFormatted === void 0 ? void 0 : cvcValueFormatted.replace(reg, '$1');
        this._rendererTimeout = window === null || window === void 0 ? void 0 : window.setTimeout(() => {
            this._ngControl.viewToModelUpdate(cvcmask);
            this._ngControl.valueAccessor.writeValue(cvcmask);
        }, TIMEOUT_SECONDS);
    }
    /**
     * Registers a callback function that is called when the control's value changes in the UI.
     * @param onChange - {fn}
     */
    registerOnChange(onChange) {
        this.onChange = onChange;
    }
    /**
     * Registers a callback function that is called by the forms API on initialization to update the form model on blur.
     * @param onTouched - {fn}
     */
    registerOnTouched(onTouched) {
        this.onTouched = onTouched;
    }
    /**
     * set element ref as disabled
     * @param isDisabled - set element as disabled
     */
    setDisabledState(isDisabled) {
        this._renderer.setProperty(this._el, 'disabled', isDisabled);
        this._control.disable({ onlySelf: true });
        this._ngControl.valueAccessor.setDisabledState(isDisabled);
    }
    /**
     * Set the mask size based on card type (from card BIN)
     */
    _setCardCvcMask() {
        this._cardTypeService.cardType
            .pipe(takeUntil(this._destroy$), map((type) => {
            var _a;
            this._cvvMaskLength = (_a = type === null || type === void 0 ? void 0 : type.code) === null || _a === void 0 ? void 0 : _a.length;
            return type;
        }))
            .subscribe();
    }
    /**
     * setup input with some attrs for better user experience
     */
    _setupInput() {
        this._renderer.setProperty(this._el, 'autoComplete', 'cc-csc');
        this._renderer.setProperty(this._el, 'aria-label', 'Card CVC');
        this._renderer.setProperty(this._el, 'id', 'cccvc');
        this._renderer.setProperty(this._el, 'name', 'cccvc');
        this._renderer.setProperty(this._el, 'type', 'tel');
    }
}
CardCvvDirective.decorators = [
    { type: Directive, args: [{
                selector: 'input[formControlName][jstCardCvv],input[formControl][jstCardCvv]',
                host: {
                    '(blur)': 'onTouched()',
                },
            },] }
];
CardCvvDirective.ctorParameters = () => [
    { type: Renderer2 },
    { type: ElementRef },
    { type: NgControl, decorators: [{ type: Optional }] },
    { type: CardTypeService }
];
CardCvvDirective.propDecorators = {
    onCardChanges: [{ type: HostListener, args: ['input', ['$event'],] }],
    onCardBlur: [{ type: HostListener, args: ['blur', ['$event'],] }],
    onKeyDown: [{ type: HostListener, args: ['keydown', ['$event'],] }]
};
//# sourceMappingURL=card-cvv.directive.js.map