import { Directive, ElementRef, HostListener, Optional, Renderer2, } from '@angular/core';
import { NgControl } from '@angular/forms';
import { CardTypeService } from '../services/card-type.service';
import utils, { TIMEOUT_SECONDS } from '../utils';
/**
 * Directive to format and validate a card expiry date
 */
export class CardExpireDateDirective {
    constructor(_renderer, _elementRef, _ngControl, _cardTypeService) {
        this._renderer = _renderer;
        this._elementRef = _elementRef;
        this._ngControl = _ngControl;
        this._cardTypeService = _cardTypeService;
        this.onChange = () => { };
        this.onTouched = () => { };
    }
    ngOnInit() {
        this._setupInput();
        this._cardTypeService.setCardExpireRef(this._elementRef);
    }
    ngOnDestroy() {
        if (this._rendererTimeout) {
            window === null || window === void 0 ? void 0 : window.clearTimeout(this._rendererTimeout);
        }
    }
    /**
     * get the current html input from element ref
     * @returns The native element from ref
     * @private
     */
    get _el() {
        return this._elementRef.nativeElement;
    }
    /**
     * get the control of the current element ref
     * @returns return the control of element
     * @private
     */
    get _control() {
        return this._ngControl.control;
    }
    /**
     * listener the input event from html input element
     * @param event - the keyboard event
     */
    onCardChanges(event) {
        this.writeValue(event);
    }
    /**
     * listener the input blur event to update input mask
     * @param event - the keyboard event
     */
    onCardBlur(event) {
        this.writeValue(event);
    }
    /**
     * Listener the keydown to check when backspace is pressed
     * @param event {KeyboardEvent} - The keyboard event
     */
    onKeyDown(event) {
        var _a, _b, _c, _d;
        const value = (_a = event === null || event === void 0 ? void 0 : event.target) === null || _a === void 0 ? void 0 : _a.value;
        if (event.key.toLowerCase() === utils.BACKSPACE_KEY_CODE && !utils.isValue(value)) {
            // focus into card number field case expiry date has empty
            (_d = (_c = (_b = this._cardTypeService) === null || _b === void 0 ? void 0 : _b.cardNumberRef) === null || _c === void 0 ? void 0 : _c.nativeElement) === null || _d === void 0 ? void 0 : _d.focus();
        }
    }
    /**
     * Writes a new value to the element.
     * @param rawValue - value from input
     */
    writeValue(rawValue) {
        const expireValue = utils.formatters.formatExpiry(rawValue);
        this._rendererTimeout = window === null || window === void 0 ? void 0 : window.setTimeout(() => {
            this._ngControl.viewToModelUpdate(expireValue);
            this._ngControl.valueAccessor.writeValue(expireValue);
        }, TIMEOUT_SECONDS);
    }
    /**
     * Registers a callback function that is called when the control's value changes in the UI.
     * @param onChange - {fn}
     */
    registerOnChange(onChange) {
        this.onChange = onChange;
    }
    /**
     * Registers a callback function that is called by the forms API on initialization to update the form model on blur.
     * @param onTouched - {fn}
     */
    registerOnTouched(onTouched) {
        this.onTouched = onTouched;
    }
    /**
     * set element ref as disabled
     * @param isDisabled - set element as disabled
     */
    setDisabledState(isDisabled) {
        this._renderer.setProperty(this._el, 'disabled', isDisabled);
        this._control.disable({ onlySelf: true });
        this._ngControl.valueAccessor.setDisabledState(isDisabled);
    }
    /**
     * setup input with some attrs for better user experience
     */
    _setupInput() {
        this._renderer.setProperty(this._el, 'autoComplete', 'cc-exp');
        this._renderer.setProperty(this._el, 'aria-label', 'Expiry date in format MM YY');
        this._renderer.setProperty(this._el, 'id', 'ccExpireDate');
        this._renderer.setProperty(this._el, 'name', 'ccExpireDate');
        this._renderer.setProperty(this._el, 'type', 'tel');
    }
}
CardExpireDateDirective.decorators = [
    { type: Directive, args: [{
                selector: 'input[jstCardExpireDate][formControlName],input[formControl][jstCardExpireDate]',
                host: {
                    '(blur)': 'onTouched()',
                },
            },] }
];
CardExpireDateDirective.ctorParameters = () => [
    { type: Renderer2 },
    { type: ElementRef },
    { type: NgControl, decorators: [{ type: Optional }] },
    { type: CardTypeService }
];
CardExpireDateDirective.propDecorators = {
    onCardChanges: [{ type: HostListener, args: ['input', ['$event'],] }],
    onCardBlur: [{ type: HostListener, args: ['blur', ['$event'],] }],
    onKeyDown: [{ type: HostListener, args: ['keydown', ['$event'],] }]
};
//# sourceMappingURL=card-expire-date.directive.js.map