import { Directive, ElementRef, HostListener, Optional, Renderer2, } from '@angular/core';
import { NgControl } from '@angular/forms';
import clearSpaces from '../utils/clear-spaces';
import utils, { TIMEOUT_SECONDS } from '../utils';
import { CardTypeService } from '../services/card-type.service';
/**
 * Directive to format and validate a card number input type
 */
export class CardNumberDirective {
    constructor(_renderer, _elementRef, _ngControl, _cardTypeService) {
        this._renderer = _renderer;
        this._elementRef = _elementRef;
        this._ngControl = _ngControl;
        this._cardTypeService = _cardTypeService;
        this.onChange = () => { };
        this.onTouched = () => { };
    }
    ngOnInit() {
        this._setupInput();
        this._cardTypeService.setCardNumberRef(this._elementRef);
    }
    ngOnDestroy() {
        if (this._rendererTimeout) {
            window === null || window === void 0 ? void 0 : window.clearTimeout(this._rendererTimeout);
        }
    }
    /**
     * get the current html input from element ref
     * @returns The native element from ref
     * @private
     */
    get _el() {
        return this._elementRef.nativeElement;
    }
    /**
     * get the control of the current element ref
     * @returns return the control of element
     * @private
     */
    get _control() {
        return this._ngControl.control;
    }
    /**
     * listener the input event from html input element
     * @param event - the keyboard event
     */
    onCardChanges(event) {
        this.writeValue(event);
    }
    /**
     * listener the input blur event to update input mask
     * @param event - the keyboard event
     */
    onCardBlur(event) {
        this.writeValue(event);
    }
    /**
     * Writes a new value to the element.
     * @param rawValue - value from input
     */
    writeValue(rawValue) {
        var _a;
        const cardNumber = ((_a = rawValue === null || rawValue === void 0 ? void 0 : rawValue.target) === null || _a === void 0 ? void 0 : _a.value) || '';
        const cardNumberFormatted = clearSpaces(cardNumber);
        const cardValue = utils.formatters.formatCardNumber(cardNumberFormatted);
        const cardType = utils.cardTypes.getCardTypeByValue(cardNumberFormatted);
        const cardNumberError = utils.validators.getCardNumberError(cardValue, null);
        // update card type value
        this._cardTypeService.setCardType(cardType);
        // check errors
        if (!cardNumberError) {
            this._checkCardNumberMaxLength(rawValue);
            this._control.setValidators(null);
        }
        // update ng control value
        this._rendererTimeout = window === null || window === void 0 ? void 0 : window.setTimeout(() => {
            this._ngControl.viewToModelUpdate(cardValue);
            this._ngControl.valueAccessor.writeValue(cardValue);
        }, TIMEOUT_SECONDS);
    }
    /**
     * Registers a callback function that is called when the control's value changes in the UI.
     * @param onChange - {fn}
     */
    registerOnChange(onChange) {
        this.onChange = onChange;
    }
    /**
     * Registers a callback function that is called by the forms API on initialization to update the form model on blur.
     * @param onTouched - {fn}
     */
    registerOnTouched(onTouched) {
        this.onTouched = onTouched;
    }
    /**
     * set element ref as disabled
     * @param isDisabled - set element as disabled
     */
    setDisabledState(isDisabled) {
        this._renderer.setProperty(this._el, 'disabled', isDisabled);
        this._control.disable({ onlySelf: true });
        this._ngControl.valueAccessor.setDisabledState(isDisabled);
    }
    /**
     * Check if card number has reached max value and focus into expire date field
     * @param event {KeyboardEvent} - Keyboard event from the card number input
     */
    _checkCardNumberMaxLength(event) {
        var _a, _b, _c;
        const cardNumber = ((_a = event === null || event === void 0 ? void 0 : event.target) === null || _a === void 0 ? void 0 : _a.value) || '';
        const cardNumberFormatted = clearSpaces(cardNumber);
        const hasReachedMaxLength = utils.validators.hasCardNumberReachedMaxLength(cardNumberFormatted);
        if (hasReachedMaxLength) {
            event.preventDefault();
            (_c = (_b = this._cardTypeService.cardExpiryRef) === null || _b === void 0 ? void 0 : _b.nativeElement) === null || _c === void 0 ? void 0 : _c.focus();
        }
    }
    /**
     * setup input with some attrs for better user experience
     */
    _setupInput() {
        this._renderer.setProperty(this._el, 'autoComplete', 'cc-number');
        this._renderer.setProperty(this._el, 'aria-label', 'Card Number');
        this._renderer.setProperty(this._el, 'id', 'ccCardNumber');
        this._renderer.setProperty(this._el, 'name', 'ccCardNumber');
        this._renderer.setProperty(this._el, 'type', 'tel');
    }
}
CardNumberDirective.decorators = [
    { type: Directive, args: [{
                selector: 'input[jstCardNumber][formControlName],input[formControl][jstCardNumber]',
                host: {
                    '(blur)': 'onTouched()',
                },
            },] }
];
CardNumberDirective.ctorParameters = () => [
    { type: Renderer2 },
    { type: ElementRef },
    { type: NgControl, decorators: [{ type: Optional }] },
    { type: CardTypeService }
];
CardNumberDirective.propDecorators = {
    onCardChanges: [{ type: HostListener, args: ['input', ['$event'],] }],
    onCardBlur: [{ type: HostListener, args: ['blur', ['$event'],] }]
};
//# sourceMappingURL=card-number.directive.js.map