import { Injectable, Directive, Renderer2, ElementRef, Optional, HostListener, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { NgControl } from '@angular/forms';
import { BACKSPACE, ENTER } from '@angular/cdk/keycodes';
import { BehaviorSubject, Subject } from 'rxjs';
import { takeUntil, map } from 'rxjs/operators';

/**
 * @description returns a value without spaces
 * @param value {string} - any string
 */
function clearSpaces (value) {
    if (!value) {
        return;
    }
    return String(value).replace(/\s/g, '');
}

const DEFAULT_CVC_LENGTH = 3;
const DEFAULT_ZIP_LENGTH = 5;
const DEFAULT_CARD_FORMAT = /(\d{1,4})/g;
/**
 * Card Types per BIN
 * @internal
 */
const CARD_TYPES = [
    {
        displayName: 'Visa',
        type: 'visa',
        format: DEFAULT_CARD_FORMAT,
        startPattern: /^4/,
        gaps: [4, 8, 12],
        lengths: [16, 18, 19],
        code: {
            name: 'CVV',
            length: 3,
        },
    },
    {
        displayName: 'Mastercard',
        type: 'mastercard',
        format: DEFAULT_CARD_FORMAT,
        startPattern: /^(5[1-5]|677189)|^(222[1-9]|2[3-6]\d{2}|27[0-1]\d|2720)/,
        gaps: [4, 8, 12],
        lengths: [16],
        code: {
            name: 'CVC',
            length: 3,
        },
    },
    {
        displayName: 'American Express',
        type: 'amex',
        format: /(\d{1,4})(\d{1,6})?(\d{1,5})?/,
        startPattern: /^3[47]/,
        gaps: [4, 10],
        lengths: [15],
        code: {
            name: 'CID',
            length: 4,
        },
    },
    {
        displayName: 'Diners Club',
        type: 'dinersclub',
        format: DEFAULT_CARD_FORMAT,
        startPattern: /^(36|38|30[0-5])/,
        gaps: [4, 10],
        lengths: [14, 16, 19],
        code: {
            name: 'CVV',
            length: 3,
        },
    },
    {
        displayName: 'Discover',
        type: 'discover',
        format: DEFAULT_CARD_FORMAT,
        startPattern: /^(6011|65|64[4-9]|622)/,
        gaps: [4, 8, 12],
        lengths: [16, 19],
        code: {
            name: 'CID',
            length: 3,
        },
    },
    {
        displayName: 'JCB',
        type: 'jcb',
        format: DEFAULT_CARD_FORMAT,
        startPattern: /^35/,
        gaps: [4, 8, 12],
        lengths: [16, 17, 18, 19],
        code: {
            name: 'CVV',
            length: 3,
        },
    },
    {
        displayName: 'UnionPay',
        type: 'unionpay',
        format: DEFAULT_CARD_FORMAT,
        startPattern: /^62/,
        gaps: [4, 8, 12],
        lengths: [14, 15, 16, 17, 18, 19],
        code: {
            name: 'CVN',
            length: 3,
        },
    },
    {
        displayName: 'Maestro',
        type: 'maestro',
        format: DEFAULT_CARD_FORMAT,
        startPattern: /^(5018|5020|5038|6304|6703|6708|6759|676[1-3])/,
        gaps: [4, 8, 12],
        lengths: [12, 13, 14, 15, 16, 17, 18, 19],
        code: {
            name: 'CVC',
            length: 3,
        },
    },
    {
        displayName: 'Elo',
        type: 'elo',
        format: DEFAULT_CARD_FORMAT,
        startPattern: /^(4011(78|79)|43(1274|8935)|45(1416|7393|763(1|2))|50(4175|6699|67[0-7][0-9]|9000)|627780|63(6297|6368)|650(03([^4])|04([0-9])|05(0|1)|4(0[5-9]|3[0-9]|8[5-9]|9[0-9])|5([0-2][0-9]|3[0-8])|9([2-6][0-9]|7[0-8])|541|700|720|901)|651652|655000|655021)/,
        gaps: [4, 8, 12],
        lengths: [16],
        code: {
            name: 'CVE',
            length: 3,
        },
    },
    {
        displayName: 'Hipercard',
        type: 'hipercard',
        format: DEFAULT_CARD_FORMAT,
        startPattern: /^(384100|384140|384160|606282|637095|637568|60(?!11))/,
        gaps: [4, 8, 12],
        lengths: [16],
        code: {
            name: 'CVC',
            length: 3,
        },
    },
];
/**
 * Get card type by card value
 * @param value - card number value
 * @returns object with all properties from card type
 */
const getCardTypeByValue = (value) => CARD_TYPES.filter((cardType) => cardType.startPattern.test(value))[0];
/**
 * Get card type by type (ie.: visa, elo, hipercard)
 * @param value - card type (ie.: visa, elo, hipercard, amex)
 * @returns object with all properties from card type
 */
const getCardTypeByType = (type) => CARD_TYPES.filter((cardType) => cardType.type === type)[0];

var cardTypes = /*#__PURE__*/Object.freeze({
    __proto__: null,
    DEFAULT_CVC_LENGTH: DEFAULT_CVC_LENGTH,
    DEFAULT_ZIP_LENGTH: DEFAULT_ZIP_LENGTH,
    DEFAULT_CARD_FORMAT: DEFAULT_CARD_FORMAT,
    CARD_TYPES: CARD_TYPES,
    getCardTypeByValue: getCardTypeByValue,
    getCardTypeByType: getCardTypeByType
});

/**
 * @description format card number based on card type list per card BIN
 * @param cardNumber {string} - card number form html input
 */
const formatCardNumber = (cardNumber) => {
    var _a, _b;
    const cardType = getCardTypeByValue(cardNumber);
    if (!cardType)
        return (_a = ((cardNumber === null || cardNumber === void 0 ? void 0 : cardNumber.match(/\d+/g)) || [])) === null || _a === void 0 ? void 0 : _a.join('');
    const format = cardType === null || cardType === void 0 ? void 0 : cardType.format;
    if (format === null || format === void 0 ? void 0 : format.global) {
        return (_b = ((cardNumber === null || cardNumber === void 0 ? void 0 : cardNumber.match(format)) || [])) === null || _b === void 0 ? void 0 : _b.join(' ');
    }
    if (format) {
        const execResult = format.exec(cardNumber.split(' ').join(''));
        if (execResult) {
            return execResult
                .splice(1, 3)
                .filter((x) => x)
                .join(' ');
        }
    }
    return cardNumber;
};
/**
 * @description Format expiry date based on card BIN
 * @param event {unknown} - Event from card date expiry date
 * @returns string
 */
const formatExpiry = (event) => {
    var _a, _b, _c, _d;
    const eventData = (_a = event === null || event === void 0 ? void 0 : event.nativeEvent) === null || _a === void 0 ? void 0 : _a.data;
    const prevExpiry = (_d = (_c = (_b = event === null || event === void 0 ? void 0 : event.target) === null || _b === void 0 ? void 0 : _b.value) === null || _c === void 0 ? void 0 : _c.split(' / ')) === null || _d === void 0 ? void 0 : _d.join('/');
    if (!prevExpiry)
        return null;
    let expiry = prevExpiry;
    if (/^[2-9]$/.test(expiry)) {
        expiry = `0${expiry}`;
    }
    if (prevExpiry.length === 2 && +prevExpiry > 12) {
        const [head, ...tail] = prevExpiry.split('');
        expiry = `0${head}/${tail.join('')}`;
    }
    if (/^1[/-]$/.test(expiry)) {
        return `01 / `;
    }
    expiry = expiry.match(/(\d{1,2})/g) || [];
    if ((expiry === null || expiry === void 0 ? void 0 : expiry.length) === 1) {
        if (!eventData && prevExpiry.includes('/')) {
            return expiry[0];
        }
        if (/\d{2}/.test(expiry)) {
            return `${expiry[0]} / `;
        }
    }
    if ((expiry === null || expiry === void 0 ? void 0 : expiry.length) > 2) {
        const [, month = null, year = null] = expiry.join('').match(/^(\d{2}).*(\d{2})$/) || [];
        return [month, year].join(' / ');
    }
    return expiry.join(' / ');
};

var formatters = /*#__PURE__*/Object.freeze({
    __proto__: null,
    formatCardNumber: formatCardNumber,
    formatExpiry: formatExpiry
});

const MONTH_REGEX = /(0[1-9]|1[0-2])/;
const EMPTY_CARD_NUMBER = 'Enter a card number';
const EMPTY_EXPIRY_DATE = 'Enter an expiry date';
const EMPTY_CVC = 'Enter a CVC';
const EMPTY_ZIP = 'Enter a ZIP code';
const INVALID_CARD_NUMBER = 'Card number is invalid';
const INVALID_EXPIRY_DATE = 'Expiry date is invalid';
const INVALID_CVC = 'CVC is invalid';
const MONTH_OUT_OF_RANGE = 'Expiry month must be between 01 and 12';
const YEAR_OUT_OF_RANGE = 'Expiry year cannot be in the past';
const DATE_OUT_OF_RANGE = 'Expiry date cannot be in the past';
/**
 * Check if a input value has reached max value based on card BIN
 * @param currentValue - current card number value without spaces
 * @returns if card number has reached max lenght
 */
const hasCardNumberReachedMaxLength = (currentValue) => {
    const cardType = cardTypes === null || cardTypes === void 0 ? void 0 : getCardTypeByValue(currentValue);
    return cardType && cardType.lengths.some((l) => clearSpaces(currentValue).length === l);
};
/**
 * check if value is a numeric type
 * @param e {KeyboardEvent} - Event from input
 * @returns if a value is a numeric value
 */
const isNumeric = (e) => {
    var _a;
    return (_a = /^\d*$/) === null || _a === void 0 ? void 0 : _a.test(e === null || e === void 0 ? void 0 : e.key);
};
/**
 * Validate card number luhn
 * @param cardNumber - card number without spaces
 */
const validateLuhn = (cardNumber) => {
    return (cardNumber
        .split('')
        .reverse()
        .map((digit) => parseInt(digit, 10))
        .map((digit, idx) => (idx % 2 ? digit * 2 : digit))
        .map((digit) => (digit > 9 ? (digit % 10) + 1 : digit))
        .reduce((accum, digit) => (accum += digit)) %
        10 ===
        0);
};
/**
 * Get all error messages from card number
 * @param cardNumber - card number without spaces
 * @param cardNumberValidator - fn to validate a card number
 * @param errorMessageObject - object with all error messages
 * @returns returns error message for card number
 */
const getCardNumberError = (cardNumber, cardNumberValidator, { errorMessages = {} } = {}) => {
    if (!cardNumber) {
        return errorMessages.emptyCardNumber || EMPTY_CARD_NUMBER;
    }
    const rawCardNumber = cardNumber.replace(/\s/g, '');
    const cardType = getCardTypeByValue(rawCardNumber);
    if (cardType && cardType.lengths) {
        const doesCardNumberMatchLength = cardType.lengths.includes(rawCardNumber.length);
        if (doesCardNumberMatchLength) {
            const isLuhnValid = validateLuhn(rawCardNumber);
            if (isLuhnValid) {
                if (cardNumberValidator) {
                    return cardNumberValidator({ cardNumber: rawCardNumber, cardType, errorMessages });
                }
                return;
            }
        }
    }
    return errorMessages.invalidCardNumber || INVALID_CARD_NUMBER;
};
/**
 * get all error messages from card expire date
 * @param expiryDate - expire date
 * @param expiryValidator - fn to validade a card expire date
 * @param errorMessageObject - object with all error message
 * @returns error message for date validation
 */
const getExpiryDateError = (expiryDate, expiryValidator, { errorMessages = {} } = {}) => {
    if (!expiryDate) {
        return errorMessages.emptyExpiryDate || EMPTY_EXPIRY_DATE;
    }
    const rawExpiryDate = expiryDate.replace(' / ', '').replace('/', '');
    if (rawExpiryDate.length === 4) {
        const month = rawExpiryDate.slice(0, 2);
        const year = `20${rawExpiryDate.slice(2, 4)}`;
        if (!MONTH_REGEX.test(month)) {
            return errorMessages.monthOutOfRange || MONTH_OUT_OF_RANGE;
        }
        if (parseInt(year) < new Date().getFullYear()) {
            return errorMessages.yearOutOfRange || YEAR_OUT_OF_RANGE;
        }
        if (parseInt(year) === new Date().getFullYear() &&
            parseInt(month) < new Date().getMonth() + 1) {
            return errorMessages.dateOutOfRange || DATE_OUT_OF_RANGE;
        }
        if (expiryValidator) {
            return expiryValidator({ expiryDate: { month, year }, errorMessages });
        }
        return;
    }
    return errorMessages.invalidExpiryDate || INVALID_EXPIRY_DATE;
};
/**
 * get all error messages from card cvc
 * @param cvc - cvc number
 * @param cvcValidator - validator fn for cvc
 * @param errorMessageObject - object with all error message
 * @returns error message for  cvc
 */
const getCVCError = (cvc, cvcValidator, { cardType, errorMessages = {} } = {}) => {
    if (!cvc) {
        return errorMessages.emptyCVC || EMPTY_CVC;
    }
    if (cvc.length < 3) {
        return errorMessages.invalidCVC || INVALID_CVC;
    }
    if (cardType && cvc.length !== cardType.code.length) {
        return errorMessages.invalidCVC || INVALID_CVC;
    }
    if (cvcValidator) {
        return cvcValidator({ cvc, cardType, errorMessages });
    }
    return;
};
/**
 * get all error messages from card cvc
 * @param zip - zip code
 * @param errorMessageObject - object with all error message
 * @returns error message for zip code
 */
const getZIPError = (zip, { errorMessages = {} } = {}) => {
    if (!zip) {
        return errorMessages.emptyZIP || EMPTY_ZIP;
    }
    return;
};

var validators = /*#__PURE__*/Object.freeze({
    __proto__: null,
    EMPTY_CARD_NUMBER: EMPTY_CARD_NUMBER,
    EMPTY_EXPIRY_DATE: EMPTY_EXPIRY_DATE,
    EMPTY_CVC: EMPTY_CVC,
    EMPTY_ZIP: EMPTY_ZIP,
    INVALID_CARD_NUMBER: INVALID_CARD_NUMBER,
    INVALID_EXPIRY_DATE: INVALID_EXPIRY_DATE,
    INVALID_CVC: INVALID_CVC,
    MONTH_OUT_OF_RANGE: MONTH_OUT_OF_RANGE,
    YEAR_OUT_OF_RANGE: YEAR_OUT_OF_RANGE,
    DATE_OUT_OF_RANGE: DATE_OUT_OF_RANGE,
    hasCardNumberReachedMaxLength: hasCardNumberReachedMaxLength,
    isNumeric: isNumeric,
    validateLuhn: validateLuhn,
    getCardNumberError: getCardNumberError,
    getExpiryDateError: getExpiryDateError,
    getCVCError: getCVCError,
    getZIPError: getZIPError
});

// kEYCODES FOR BACKSPACE KEY
const BACKSPACE_KEY_CODE = 'backspace';
const BACKSPACE_KEY_NUMBER = BACKSPACE;
// kEYCODES FOR ENTER KEY
const ENTER_KEY_CODE = 'enter';
const ENTER_KEY_NUMBER = ENTER;
/**
 * Timeout seconds to update element value
 * @internal
 */
const TIMEOUT_SECONDS = 0;
/**
 * check if a value is valid
 * @param value - value to check if has value
 */
function isValue(value) {
    return !(value == null || value === '' || value !== value);
}
var utils = {
    cardTypes,
    formatters,
    validators,
    isValue,
    BACKSPACE_KEY_CODE,
    BACKSPACE_KEY_NUMBER,
    ENTER_KEY_CODE,
    ENTER_KEY_NUMBER,
};

class CardTypeService {
    constructor() {
        this._cardType$ = new BehaviorSubject(undefined);
    }
    /**
     * Getter for card type observable
     */
    get cardType() {
        return this._cardType$.asObservable();
    }
    /**
     * Getter for card number input ref
     */
    get cardNumberRef() {
        return this._cardNumberRef;
    }
    /**
     * Getter for card expiry date input ref
     */
    get cardExpiryRef() {
        return this._cardExpiryRef;
    }
    /**
     * Getter for card CVC input ref
     */
    get cardCVVRef() {
        return this._cardCVVRef;
    }
    /**
     * Update card type object with card type from card input field.
     * @param type - card type object
     */
    setCardType(type) {
        this._cardType$.next(type);
    }
    /**
     * update value of element ref for the Card Number
     * @param el - ElementRef of the Card Number
     * @template E - Type of HTML Element
     */
    setCardNumberRef(el) {
        this._cardNumberRef = el;
    }
    /**
     * update value of element ref for the Date Expire Input
     * @param el - ElementRef of the Card Expire
     * @template E - Type of HTML Element
     */
    setCardExpireRef(el) {
        this._cardExpiryRef = el;
    }
    /**
     * update value of element ref for the CVV Input
     * @param el - ElementRef of the Card CVV
     * @template E - Type of HTML Element
     */
    setCardCVVRef(el) {
        this._cardCVVRef = el;
    }
}
CardTypeService.decorators = [
    { type: Injectable }
];
CardTypeService.ctorParameters = () => [];

/**
 * Directive to format and validate a card number input type
 */
class CardNumberDirective {
    constructor(_renderer, _elementRef, _ngControl, _cardTypeService) {
        this._renderer = _renderer;
        this._elementRef = _elementRef;
        this._ngControl = _ngControl;
        this._cardTypeService = _cardTypeService;
        this.onChange = () => { };
        this.onTouched = () => { };
    }
    ngOnInit() {
        this._setupInput();
        this._cardTypeService.setCardNumberRef(this._elementRef);
    }
    ngOnDestroy() {
        if (this._rendererTimeout) {
            window === null || window === void 0 ? void 0 : window.clearTimeout(this._rendererTimeout);
        }
    }
    /**
     * get the current html input from element ref
     * @returns The native element from ref
     * @private
     */
    get _el() {
        return this._elementRef.nativeElement;
    }
    /**
     * get the control of the current element ref
     * @returns return the control of element
     * @private
     */
    get _control() {
        return this._ngControl.control;
    }
    /**
     * listener the input event from html input element
     * @param event - the keyboard event
     */
    onCardChanges(event) {
        this.writeValue(event);
    }
    /**
     * listener the input blur event to update input mask
     * @param event - the keyboard event
     */
    onCardBlur(event) {
        this.writeValue(event);
    }
    /**
     * Writes a new value to the element.
     * @param rawValue - value from input
     */
    writeValue(rawValue) {
        var _a;
        const cardNumber = ((_a = rawValue === null || rawValue === void 0 ? void 0 : rawValue.target) === null || _a === void 0 ? void 0 : _a.value) || '';
        const cardNumberFormatted = clearSpaces(cardNumber);
        const cardValue = utils.formatters.formatCardNumber(cardNumberFormatted);
        const cardType = utils.cardTypes.getCardTypeByValue(cardNumberFormatted);
        const cardNumberError = utils.validators.getCardNumberError(cardValue, null);
        // update card type value
        this._cardTypeService.setCardType(cardType);
        // check errors
        if (!cardNumberError) {
            this._checkCardNumberMaxLength(rawValue);
            this._control.setValidators(null);
        }
        // update ng control value
        this._rendererTimeout = window === null || window === void 0 ? void 0 : window.setTimeout(() => {
            this._ngControl.viewToModelUpdate(cardValue);
            this._ngControl.valueAccessor.writeValue(cardValue);
        }, TIMEOUT_SECONDS);
    }
    /**
     * Registers a callback function that is called when the control's value changes in the UI.
     * @param onChange - {fn}
     */
    registerOnChange(onChange) {
        this.onChange = onChange;
    }
    /**
     * Registers a callback function that is called by the forms API on initialization to update the form model on blur.
     * @param onTouched - {fn}
     */
    registerOnTouched(onTouched) {
        this.onTouched = onTouched;
    }
    /**
     * set element ref as disabled
     * @param isDisabled - set element as disabled
     */
    setDisabledState(isDisabled) {
        this._renderer.setProperty(this._el, 'disabled', isDisabled);
        this._control.disable({ onlySelf: true });
        this._ngControl.valueAccessor.setDisabledState(isDisabled);
    }
    /**
     * Check if card number has reached max value and focus into expire date field
     * @param event {KeyboardEvent} - Keyboard event from the card number input
     */
    _checkCardNumberMaxLength(event) {
        var _a, _b, _c;
        const cardNumber = ((_a = event === null || event === void 0 ? void 0 : event.target) === null || _a === void 0 ? void 0 : _a.value) || '';
        const cardNumberFormatted = clearSpaces(cardNumber);
        const hasReachedMaxLength = utils.validators.hasCardNumberReachedMaxLength(cardNumberFormatted);
        if (hasReachedMaxLength) {
            event.preventDefault();
            (_c = (_b = this._cardTypeService.cardExpiryRef) === null || _b === void 0 ? void 0 : _b.nativeElement) === null || _c === void 0 ? void 0 : _c.focus();
        }
    }
    /**
     * setup input with some attrs for better user experience
     */
    _setupInput() {
        this._renderer.setProperty(this._el, 'autoComplete', 'cc-number');
        this._renderer.setProperty(this._el, 'aria-label', 'Card Number');
        this._renderer.setProperty(this._el, 'id', 'ccCardNumber');
        this._renderer.setProperty(this._el, 'name', 'ccCardNumber');
        this._renderer.setProperty(this._el, 'type', 'tel');
    }
}
CardNumberDirective.decorators = [
    { type: Directive, args: [{
                selector: 'input[jstCardNumber][formControlName],input[formControl][jstCardNumber]',
                host: {
                    '(blur)': 'onTouched()',
                },
            },] }
];
CardNumberDirective.ctorParameters = () => [
    { type: Renderer2 },
    { type: ElementRef },
    { type: NgControl, decorators: [{ type: Optional }] },
    { type: CardTypeService }
];
CardNumberDirective.propDecorators = {
    onCardChanges: [{ type: HostListener, args: ['input', ['$event'],] }],
    onCardBlur: [{ type: HostListener, args: ['blur', ['$event'],] }]
};

/**
 * Directive to format and validate a card expiry date
 */
class CardExpireDateDirective {
    constructor(_renderer, _elementRef, _ngControl, _cardTypeService) {
        this._renderer = _renderer;
        this._elementRef = _elementRef;
        this._ngControl = _ngControl;
        this._cardTypeService = _cardTypeService;
        this.onChange = () => { };
        this.onTouched = () => { };
    }
    ngOnInit() {
        this._setupInput();
        this._cardTypeService.setCardExpireRef(this._elementRef);
    }
    ngOnDestroy() {
        if (this._rendererTimeout) {
            window === null || window === void 0 ? void 0 : window.clearTimeout(this._rendererTimeout);
        }
    }
    /**
     * get the current html input from element ref
     * @returns The native element from ref
     * @private
     */
    get _el() {
        return this._elementRef.nativeElement;
    }
    /**
     * get the control of the current element ref
     * @returns return the control of element
     * @private
     */
    get _control() {
        return this._ngControl.control;
    }
    /**
     * listener the input event from html input element
     * @param event - the keyboard event
     */
    onCardChanges(event) {
        this.writeValue(event);
    }
    /**
     * listener the input blur event to update input mask
     * @param event - the keyboard event
     */
    onCardBlur(event) {
        this.writeValue(event);
    }
    /**
     * Listener the keydown to check when backspace is pressed
     * @param event {KeyboardEvent} - The keyboard event
     */
    onKeyDown(event) {
        var _a, _b, _c, _d;
        const value = (_a = event === null || event === void 0 ? void 0 : event.target) === null || _a === void 0 ? void 0 : _a.value;
        if (event.key.toLowerCase() === utils.BACKSPACE_KEY_CODE && !utils.isValue(value)) {
            // focus into card number field case expiry date has empty
            (_d = (_c = (_b = this._cardTypeService) === null || _b === void 0 ? void 0 : _b.cardNumberRef) === null || _c === void 0 ? void 0 : _c.nativeElement) === null || _d === void 0 ? void 0 : _d.focus();
        }
    }
    /**
     * Writes a new value to the element.
     * @param rawValue - value from input
     */
    writeValue(rawValue) {
        const expireValue = utils.formatters.formatExpiry(rawValue);
        this._rendererTimeout = window === null || window === void 0 ? void 0 : window.setTimeout(() => {
            this._ngControl.viewToModelUpdate(expireValue);
            this._ngControl.valueAccessor.writeValue(expireValue);
        }, TIMEOUT_SECONDS);
    }
    /**
     * Registers a callback function that is called when the control's value changes in the UI.
     * @param onChange - {fn}
     */
    registerOnChange(onChange) {
        this.onChange = onChange;
    }
    /**
     * Registers a callback function that is called by the forms API on initialization to update the form model on blur.
     * @param onTouched - {fn}
     */
    registerOnTouched(onTouched) {
        this.onTouched = onTouched;
    }
    /**
     * set element ref as disabled
     * @param isDisabled - set element as disabled
     */
    setDisabledState(isDisabled) {
        this._renderer.setProperty(this._el, 'disabled', isDisabled);
        this._control.disable({ onlySelf: true });
        this._ngControl.valueAccessor.setDisabledState(isDisabled);
    }
    /**
     * setup input with some attrs for better user experience
     */
    _setupInput() {
        this._renderer.setProperty(this._el, 'autoComplete', 'cc-exp');
        this._renderer.setProperty(this._el, 'aria-label', 'Expiry date in format MM YY');
        this._renderer.setProperty(this._el, 'id', 'ccExpireDate');
        this._renderer.setProperty(this._el, 'name', 'ccExpireDate');
        this._renderer.setProperty(this._el, 'type', 'tel');
    }
}
CardExpireDateDirective.decorators = [
    { type: Directive, args: [{
                selector: 'input[jstCardExpireDate][formControlName],input[formControl][jstCardExpireDate]',
                host: {
                    '(blur)': 'onTouched()',
                },
            },] }
];
CardExpireDateDirective.ctorParameters = () => [
    { type: Renderer2 },
    { type: ElementRef },
    { type: NgControl, decorators: [{ type: Optional }] },
    { type: CardTypeService }
];
CardExpireDateDirective.propDecorators = {
    onCardChanges: [{ type: HostListener, args: ['input', ['$event'],] }],
    onCardBlur: [{ type: HostListener, args: ['blur', ['$event'],] }],
    onKeyDown: [{ type: HostListener, args: ['keydown', ['$event'],] }]
};

/**
 * Directive to format and validate a card CVC
 */
class CardCvvDirective {
    constructor(_renderer, _elementRef, _ngControl, _cardTypeService) {
        this._renderer = _renderer;
        this._elementRef = _elementRef;
        this._ngControl = _ngControl;
        this._cardTypeService = _cardTypeService;
        this.onChange = () => { };
        this.onTouched = () => { };
        /**
         * attr for stop all subscribes
         */
        this._destroy$ = new Subject();
    }
    ngOnInit() {
        this._setupInput();
        this._cardTypeService.setCardCVVRef(this._elementRef);
    }
    ngAfterViewInit() {
        this._setCardCvcMask();
    }
    ngOnDestroy() {
        this._destroy$.next();
        this._destroy$.complete();
        if (this._rendererTimeout) {
            window === null || window === void 0 ? void 0 : window.clearTimeout(this._rendererTimeout);
        }
    }
    /**
     * get the current html input from element ref
     * @returns The native element from ref
     * @private
     */
    get _el() {
        return this._elementRef.nativeElement;
    }
    /**
     * get the control of the current element ref
     * @returns return the control of element
     * @private
     */
    get _control() {
        return this._ngControl.control;
    }
    /**
     * listener the input event from html input element
     * @param event - the keyboard event
     */
    onCardChanges(event) {
        this.writeValue(event);
    }
    /**
     * listener the input blur event to update input mask
     * @param event - the keyboard event
     */
    onCardBlur(event) {
        this.writeValue(event);
    }
    /**
     * Listener the keydown to check when backspace is pressed
     * @param event {KeyboardEvent} - The keyboard event
     */
    onKeyDown(event) {
        var _a, _b, _c, _d;
        const value = (_a = event === null || event === void 0 ? void 0 : event.target) === null || _a === void 0 ? void 0 : _a.value;
        if (event.key.toLowerCase() === utils.BACKSPACE_KEY_CODE && !utils.isValue(value)) {
            // focus into card number field case expiry date is empty
            (_d = (_c = (_b = this._cardTypeService) === null || _b === void 0 ? void 0 : _b.cardExpiryRef) === null || _c === void 0 ? void 0 : _c.nativeElement) === null || _d === void 0 ? void 0 : _d.focus();
        }
    }
    /**
     * Writes a new value to the element.
     * @param rawValue - value from input
     */
    writeValue(rawValue) {
        var _a;
        const cvcValue = ((_a = rawValue.target) === null || _a === void 0 ? void 0 : _a.value) || '';
        const cvcValueFormatted = clearSpaces(cvcValue);
        const reg = new RegExp(`([0-9]{${this._cvvMaskLength}})`, 'g');
        const cvcmask = cvcValueFormatted === null || cvcValueFormatted === void 0 ? void 0 : cvcValueFormatted.replace(reg, '$1');
        this._rendererTimeout = window === null || window === void 0 ? void 0 : window.setTimeout(() => {
            this._ngControl.viewToModelUpdate(cvcmask);
            this._ngControl.valueAccessor.writeValue(cvcmask);
        }, TIMEOUT_SECONDS);
    }
    /**
     * Registers a callback function that is called when the control's value changes in the UI.
     * @param onChange - {fn}
     */
    registerOnChange(onChange) {
        this.onChange = onChange;
    }
    /**
     * Registers a callback function that is called by the forms API on initialization to update the form model on blur.
     * @param onTouched - {fn}
     */
    registerOnTouched(onTouched) {
        this.onTouched = onTouched;
    }
    /**
     * set element ref as disabled
     * @param isDisabled - set element as disabled
     */
    setDisabledState(isDisabled) {
        this._renderer.setProperty(this._el, 'disabled', isDisabled);
        this._control.disable({ onlySelf: true });
        this._ngControl.valueAccessor.setDisabledState(isDisabled);
    }
    /**
     * Set the mask size based on card type (from card BIN)
     */
    _setCardCvcMask() {
        this._cardTypeService.cardType
            .pipe(takeUntil(this._destroy$), map((type) => {
            var _a;
            this._cvvMaskLength = (_a = type === null || type === void 0 ? void 0 : type.code) === null || _a === void 0 ? void 0 : _a.length;
            return type;
        }))
            .subscribe();
    }
    /**
     * setup input with some attrs for better user experience
     */
    _setupInput() {
        this._renderer.setProperty(this._el, 'autoComplete', 'cc-csc');
        this._renderer.setProperty(this._el, 'aria-label', 'Card CVC');
        this._renderer.setProperty(this._el, 'id', 'cccvc');
        this._renderer.setProperty(this._el, 'name', 'cccvc');
        this._renderer.setProperty(this._el, 'type', 'tel');
    }
}
CardCvvDirective.decorators = [
    { type: Directive, args: [{
                selector: 'input[formControlName][jstCardCvv],input[formControl][jstCardCvv]',
                host: {
                    '(blur)': 'onTouched()',
                },
            },] }
];
CardCvvDirective.ctorParameters = () => [
    { type: Renderer2 },
    { type: ElementRef },
    { type: NgControl, decorators: [{ type: Optional }] },
    { type: CardTypeService }
];
CardCvvDirective.propDecorators = {
    onCardChanges: [{ type: HostListener, args: ['input', ['$event'],] }],
    onCardBlur: [{ type: HostListener, args: ['blur', ['$event'],] }],
    onKeyDown: [{ type: HostListener, args: ['keydown', ['$event'],] }]
};

class PaymentInputsModule {
}
PaymentInputsModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                declarations: [CardNumberDirective, CardExpireDateDirective, CardCvvDirective],
                exports: [CardNumberDirective, CardExpireDateDirective, CardCvvDirective],
                providers: [CardTypeService],
            },] }
];

/**
 * @dynamic
 * Class with validators to use with Payment Input directives and reactive forms.
 * All class methods are static.
 *
 * @example
 * Example with a form group:
 * ```ts
 *   ngOnInit(): void {
 *    this.form = this.fb.group({
 *      cardNumber: [null, [PaymentInputValidators.validateCardNumber()]],
 *      cardCvc: [null, [PaymentInputValidators.validateCardExpiryDate()]],
 *      cardValidate: [null, [PaymentInputValidators.validateCVC()]],
 *    });
 *  }
 * ```
 */
class PaymentInputValidators {
    /**
     * Validate a card number
     * @returns Angular validator function
     * @static
     */
    static cardNumber() {
        return (control) => {
            const value = control === null || control === void 0 ? void 0 : control.value;
            const cardNumberError = utils.validators.getCardNumberError(value, null);
            return !cardNumberError ? null : { 'card-number': cardNumberError };
        };
    }
    /**
     * Validate a card expiry date
     * @returns Angular validator function
     * @static
     */
    static expiryDate() {
        return (control) => {
            const value = control === null || control === void 0 ? void 0 : control.value;
            const cardNumberError = utils.validators.getExpiryDateError(value, null);
            return !cardNumberError ? null : { 'card-expiry': cardNumberError };
        };
    }
    /**
     * Validate a card CVC
     * @returns Angular validator function
     * @static
     */
    static CVC() {
        return (control) => {
            const value = control === null || control === void 0 ? void 0 : control.value;
            const cardNumberError = utils.validators.getCVCError(value, null);
            return !cardNumberError ? null : { 'card-cvc': cardNumberError };
        };
    }
}

/**
 * Generated bundle index. Do not edit.
 */

export { CardCvvDirective, CardExpireDateDirective, CardNumberDirective, CardTypeService, PaymentInputValidators, PaymentInputsModule };
//# sourceMappingURL=ngx-payment-inputs.js.map
